import { CommandRegistry } from '@lumino/commands';
import * as React from 'react';
/**
 * Interface describing component properties.
 */
export interface ICommitBoxProps {
    /**
     * Jupyter App commands registry
     */
    commands: CommandRegistry;
    /**
     * Boolean indicating whether files currently exist which have changes to commit.
     */
    hasFiles: boolean;
    /**
     * Commit button label
     */
    label: string;
    /**
     * Callback to invoke in order to commit changes.
     *
     * @param msg - commit message
     * @returns a promise which commits changes
     */
    onCommit: (msg: string) => Promise<void>;
}
/**
 * Interface describing component state.
 */
export interface ICommitBoxState {
    /**
     * Commit message summary.
     */
    summary: string;
    /**
     * Commit message description.
     */
    description: string;
}
/**
 * React component for entering a commit message.
 */
export declare class CommitBox extends React.Component<ICommitBoxProps, ICommitBoxState> {
    /**
     * Returns a React component for entering a commit message.
     *
     * @param props - component properties
     * @returns React component
     */
    constructor(props: ICommitBoxProps);
    componentDidMount(): void;
    componentWillUnmount(): void;
    /**
     * Renders the component.
     *
     * @returns React element
     */
    render(): React.ReactElement;
    /**
     * Whether a commit can be performed (files are staged and summary is not empty).
     */
    private _canCommit;
    /**
     * Get keystroke configured to act as a submit action.
     */
    private _getSubmitKeystroke;
    /**
     * Callback invoked upon clicking a commit message submit button or otherwise submitting the form.
     */
    private _onCommitSubmit;
    /**
     * Callback invoked upon updating a commit message description.
     *
     * @param event - event object
     */
    private _onDescriptionChange;
    /**
     * Callback invoked upon updating a commit message summary.
     *
     * @param event - event object
     */
    private _onSummaryChange;
    /**
     * Callback invoked upon a `'keypress'` event when entering a commit message summary.
     *
     * ## Notes
     *
     * -   Prevents triggering a `'submit'` action when hitting the `ENTER` key while entering a commit message summary.
     *
     * @param event - event object
     */
    private _onSummaryKeyPress;
    /**
     * Callback invoked upon command execution activated when entering a commit message description.
     *
     * ## Notes
     *
     * -   Triggers the `'submit'` action on appropriate command (and if commit is possible)
     *
     */
    private _handleCommand;
    /**
     * Resets component state (e.g., in order to re-initialize the commit message input box).
     */
    private _reset;
}
