import { CommandRegistry } from '@lumino/commands';
import * as React from 'react';
import TextareaAutosize from 'react-textarea-autosize';
import { commitButtonClass, commitDescriptionClass, commitFormClass, commitSummaryClass } from '../style/CommitBox';
import { CommandIDs } from '../tokens';
/**
 * React component for entering a commit message.
 */
export class CommitBox extends React.Component {
    /**
     * Returns a React component for entering a commit message.
     *
     * @param props - component properties
     * @returns React component
     */
    constructor(props) {
        super(props);
        /**
         * Get keystroke configured to act as a submit action.
         */
        this._getSubmitKeystroke = () => {
            const binding = this.props.commands.keyBindings.find(binding => binding.command === CommandIDs.gitSubmitCommand);
            return binding.keys.join(' ');
        };
        /**
         * Callback invoked upon clicking a commit message submit button or otherwise submitting the form.
         */
        this._onCommitSubmit = () => {
            const msg = this.state.summary + '\n\n' + this.state.description + '\n';
            this.props.onCommit(msg);
            // NOTE: we assume here that committing changes always works and we can safely clear component state
            this._reset();
        };
        /**
         * Callback invoked upon updating a commit message description.
         *
         * @param event - event object
         */
        this._onDescriptionChange = (event) => {
            this.setState({
                description: event.target.value
            });
        };
        /**
         * Callback invoked upon updating a commit message summary.
         *
         * @param event - event object
         */
        this._onSummaryChange = (event) => {
            this.setState({
                summary: event.target.value
            });
        };
        /**
         * Callback invoked upon a `'keypress'` event when entering a commit message summary.
         *
         * ## Notes
         *
         * -   Prevents triggering a `'submit'` action when hitting the `ENTER` key while entering a commit message summary.
         *
         * @param event - event object
         */
        this._onSummaryKeyPress = (event) => {
            if (event.key === 'Enter') {
                event.preventDefault();
            }
        };
        /**
         * Callback invoked upon command execution activated when entering a commit message description.
         *
         * ## Notes
         *
         * -   Triggers the `'submit'` action on appropriate command (and if commit is possible)
         *
         */
        this._handleCommand = (_, commandArgs) => {
            if (commandArgs.id === CommandIDs.gitSubmitCommand && this._canCommit()) {
                this._onCommitSubmit();
            }
        };
        this.state = {
            summary: '',
            description: ''
        };
    }
    componentDidMount() {
        this.props.commands.commandExecuted.connect(this._handleCommand);
    }
    componentWillUnmount() {
        this.props.commands.commandExecuted.disconnect(this._handleCommand);
    }
    /**
     * Renders the component.
     *
     * @returns React element
     */
    render() {
        const disabled = !this._canCommit();
        const title = !this.props.hasFiles
            ? 'Disabled: No files are staged for commit'
            : !this.state.summary
                ? 'Disabled: No commit message summary'
                : this.props.label;
        const shortcutHint = CommandRegistry.formatKeystroke(this._getSubmitKeystroke());
        const summaryPlaceholder = 'Summary (' + shortcutHint + ' to commit)';
        return (React.createElement("form", { className: [commitFormClass, 'jp-git-CommitBox'].join(' ') },
            React.createElement("input", { className: commitSummaryClass, type: "text", placeholder: summaryPlaceholder, title: "Enter a commit message summary (a single line, preferably less than 50 characters)", value: this.state.summary, onChange: this._onSummaryChange, onKeyPress: this._onSummaryKeyPress }),
            React.createElement(TextareaAutosize, { className: commitDescriptionClass, minRows: 5, placeholder: "Description (optional)", title: "Enter a commit message description", value: this.state.description, onChange: this._onDescriptionChange }),
            React.createElement("input", { className: commitButtonClass, type: "button", title: title, value: this.props.label, disabled: disabled, onClick: this._onCommitSubmit })));
    }
    /**
     * Whether a commit can be performed (files are staged and summary is not empty).
     */
    _canCommit() {
        return !!(this.props.hasFiles && this.state.summary);
    }
    /**
     * Resets component state (e.g., in order to re-initialize the commit message input box).
     */
    _reset() {
        this.setState({
            summary: '',
            description: ''
        });
    }
}
//# sourceMappingURL=CommitBox.js.map