import { Mode } from '@jupyterlab/codemirror';
import { PromiseDelegate } from '@lumino/coreutils';
import { Widget } from '@lumino/widgets';
import { mergeView } from './mergeview';
/**
 * Diff callback to be registered for plain-text files.
 *
 * @param model Diff model
 * @param toolbar MainAreaWidget toolbar
 * @returns PlainText diff widget
 */
export const createPlainTextDiff = async (model, toolbar) => {
    const widget = new PlainTextDiff(model);
    await widget.ready;
    return widget;
};
/**
 * Plain Text Diff widget
 */
export class PlainTextDiff extends Widget {
    constructor(model) {
        super({
            node: PlainTextDiff.createNode(model.reference.label, model.challenger.label)
        });
        this._reference = null;
        this._challenger = null;
        const getReady = new PromiseDelegate();
        this._isReady = getReady.promise;
        this._container = this.node.lastElementChild;
        this._model = model;
        // Load file content early
        Promise.all([
            this._model.reference.content(),
            this._model.challenger.content()
        ])
            .then(([reference, challenger]) => {
            this._reference = reference;
            this._challenger = challenger;
            getReady.resolve();
        })
            .catch(reason => {
            this.showError(reason);
            getReady.resolve();
        });
    }
    /**
     * Promise which fulfills when the widget is ready.
     */
    get ready() {
        return this._isReady;
    }
    /**
     * Callback to create the diff widget once the widget
     * is attached so CodeMirror get proper size.
     */
    onAfterAttach() {
        this.createDiffView().catch(reason => {
            this.showError(reason);
        });
    }
    /**
     * Undo onAfterAttach
     */
    onBeforeDetach() {
        this._container.innerHTML = '';
    }
    /**
     * Refresh diff
     *
     * Note: Update the content and recompute the diff
     */
    async refresh() {
        await this.ready;
        try {
            // Clear all
            this._container.innerHTML = '';
            this._mergeView = null;
            // ENH request content only if it changed
            this.createDiffView();
        }
        catch (reason) {
            this.showError(reason);
        }
    }
    /**
     * Create wrapper node
     */
    static createNode(baseLabel, remoteLabel) {
        const head = document.createElement('div');
        head.className = 'jp-git-diff-root';
        head.innerHTML = `
    <div class="jp-git-diff-banner">
      <span>${baseLabel}</span>
      <span class="jp-spacer"></span>
      <span>${remoteLabel}</span>
    </div>
    <div class="jp-git-PlainText-diff"></div>`;
        return head;
    }
    /**
     * Create the Plain Text Diff view
     */
    async createDiffView() {
        if (!this._mergeView) {
            const mode = Mode.findByFileName(this._model.filename) ||
                Mode.findBest(this._model.filename);
            await this.ready;
            if (!this._reference) {
                this._reference = await this._model.reference.content();
            }
            if (!this._challenger) {
                this._challenger = await this._model.challenger.content();
            }
            this._mergeView = mergeView(this._container, Object.assign({ value: this._challenger, orig: this._reference, mode: mode.mime }, PlainTextDiff.getDefaultOptions()));
            this._reference = null;
            this._challenger = null;
        }
    }
    /**
     * Display an error instead of the file diff
     *
     * @param error Error object
     */
    showError(error) {
        var _a;
        console.error('Failed to load file diff.', error, (_a = error) === null || _a === void 0 ? void 0 : _a.traceback);
        const msg = (error.message || error).replace('\n', '<br />');
        this.node.innerHTML = `<p class="jp-git-diff-error">
      <span>Error Loading File Diff:</span>
      <span class="jp-git-diff-error-message">${msg}</span>
    </p>`;
    }
    static getDefaultOptions() {
        // FIXME add options from settings and connect settings to update options
        return {
            lineNumbers: true,
            theme: 'jupyter',
            connect: 'align',
            collapseIdentical: true,
            readOnly: true,
            revertButtons: false
        };
    }
}
//# sourceMappingURL=PlainTextDiff.js.map